(function ($) {

  /**
   * File upload limit functions.
   *
   * @namespace
   */
  Drupal.fileUploadSizeLimit = Drupal.fileUploadSizeLimit || {

    /**
     * Client-side file input validation of file size.
     *
     * @name Drupal.fileUploadSizeLimit.uploadLimitSizeValidate
     *
     * @param {jQuery.Event} event
     *   The event triggered. For example `change.fileValidate`.
     */
    uploadLimitSizeValidate(event) {
      event.preventDefault();
      // Remove any previous errors.
      $('.file-upload-js-error').remove();

      // Define variables.
      var single_file_size = event.data.single_size;
      var multiple_limit_type = event.data.multiple_limit_type;
      var multiple_file_size = event.data.multiple_size;

      // Check uploaded files if they are exists and single_file_size is defined.
      if (!navigator.userAgent.match(/MSIE\s(?!10.0)/)) {
        if (this.value.length > 0 && this.files.length > 0) {
          // Calculate the size of the uploaded files.
          var total_upload_size = 0;
          var upload_input = this;
          $.each(this.files, function (ind, val) {
            // Check the single file size limit.
            if (val.size > single_file_size) {
              var error = Drupal.t('The size of the uploaded file (%file_size MB) exceeds the maximum allowed size for a single file of %single_file_max_size MB.', {
                '%file_size': parseFloat((val.size / (1024*1024)).toFixed(2)),
                '%single_file_max_size': parseFloat((single_file_size / (1024*1024)).toFixed(2)),
              });
              Drupal.fileUploadSizeLimit.showError(event, upload_input, error);
            }

            // Sum the total file upload size.
            total_upload_size += val.size;
          });

          // Check the total multiple file size limit.
          if (multiple_limit_type !== 'unlimited' && (total_upload_size > multiple_file_size)) {
            var error = Drupal.t('The total size of the uploaded file(s) (%total_upload_size MB) exceeds the maximum allowed size of %allowed_total_size MB.', {
              '%total_upload_size': parseFloat((total_upload_size / (1024*1024)).toFixed(2)),
              '%allowed_total_size': parseFloat((multiple_file_size / (1024*1024)).toFixed(2)),
            });
            Drupal.fileUploadSizeLimit.showError(event, upload_input, error);
          }
        }
      }
    },

    showError(event, upload_button, error) {
      $(upload_button)
          .closest('div.js-form-managed-file')
          .prepend(
              `<div class="messages messages--error file-upload-js-error" aria-live="polite">${error}</div>`,
          );
      upload_button.value = '';
      // Cancel all other change event handlers.
      event.stopImmediatePropagation();
    },
  };

  /**
   * Attach behaviors to managed file element upload fields.
   */
  Drupal.behaviors.fileUploadSizeLimitAutoAttach = {
    attach(context, settings) {
      const $context = $(context);
      let elements;

      function initFileValidation(selector) {
        $context
            .find(selector)
            .once('fileUploadSizeLimitValidate')
            .on(
                'change',
                {
                  single_size: settings.fileUploadSizeLimit.single_size,
                  multiple_limit_type: settings.fileUploadSizeLimit.multiple_limit_type,
                  multiple_size: settings.fileUploadSizeLimit.multiple_size
                },
                Drupal.fileUploadSizeLimit.uploadLimitSizeValidate,
            );
      }

      if (settings.file && settings.file.elements) {
        elements = settings.file.elements;
        Object.keys(elements).forEach(initFileValidation);
      }
    },
    detach(context, settings, trigger) {
      const $context = $(context);
      let elements;

      function removeFileValidation(selector) {
        $context
            .find(selector)
            .removeOnce('fileUploadSizeLimitValidate')
            .off('change', Drupal.fileUploadSizeLimit.uploadLimitSizeValidate);
      }

      if (trigger === 'unload' && settings.file && settings.file.elements) {
        elements = settings.file.elements;
        Object.keys(elements).forEach(removeFileValidation);
      }
    },
  };

})(jQuery);
